// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file may be used with scilab 2.0 and 3.0
// Computation times are much longer with scilab 4.0 and the results sometimes strange...
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file is used by the files example20.sci and example21.sci
////////////////////////////////////////////////////////////////////

//Functions allowing the identification of the coefficient and delay of a 
//linear differential system with delay of the form:
// x'' = a1 x' + a0 x + b u(t + R)
//See use in the file Exemple2B.sci

//Discrete integration. Trapeze method.
function inte=intDiscr(A,B,T0,n,Fr)
  n0=floor(T0*Fr);
  n1=n0+n-1;
  C=A(n0:n1).*B;
  inte=pas*((C(1)+C(n1-n0+1))/2+C(1,1:n1-n0-1)*ones(n1-n0-1,1));
endfunction

//TDEBUT Time at the begining of output file
//TFIN   Time at the end of output file
//pas = 1/Freq: saving step here 0.002
//Sortie: the output
//T11 is T1 for i=0. Should be >= TDEBUT+R.
//T2m is T2 for i=m. Should be <= TFIN.
//T Should be (much) >> |R-R0| !
//Integrals used from T_1 = T11 + DT*(i-1) to T_2 = T11 + T (for i=0:m),
//where DT=(T2m-T11-T)/(m-1).
//R0 approximation of the delay
//m number of integrals used. DT*m + T should be <= TFIN
//and m should be >=5.
//iter Number of iterations to be done with new R0 taken as the last 
//approximation computed. 

function resu=delayIdentificationOrder2(TDEBUT,TFIN,pas,Sortie,Commande,T11,T2m,T,m,R0,iter)
  R1=R0;
  DT=(T2m-T11-T)/(m-1)
  Freq=1/pas;
  for j=1:iter do 
    CC=zeros(m,5);DD=zeros(m,1);
    omega=2*%pi/T;
    SIN=sin(omega*[0:pas:T]);
    COS=cos(omega*[0:pas:T]);[w,z]=size(COS);
    UNMOINSCOS=ones(w,z)-COS;
      for i=1:m do
        T1=T11+(i-1)*DT;
        T2=T1+T;
        DD(i,1)=omega^2*intDiscr(Sortie,-COS,T1,z,Freq);
        CC(i,1)=omega*intDiscr(Sortie,SIN,T1,z,Freq);
        CC(i,2)=-intDiscr(Sortie,UNMOINSCOS,T1,z,Freq);
        CC(i,3)=-intDiscr(Commande,UNMOINSCOS,T1-R1,z,Freq);
        CC(i,4)=intDiscr(Commande,COS,T1-R1,z,Freq);
        CC(i,5)=intDiscr(Commande,SIN,T1-R1,z,Freq);
      end;
    re=lsq(CC,DD);R1=R1-0.2*asin(re(5,1))/omega;R1=floor(R1*Freq)*pas;
  end;
  resu=[re(1,1), re(2,1), re(3,1), R1];
endfunction;
