// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file has been used with scilab 3.0
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file corresponds to example 3 Table 2.
////////////////////////////////////////////////////////////////////

//We load the function used for the resolution
exec('example3A.sci');

//Beginning and end of the computations
TFIN=70;
TDEBUT=0.;

//The delays
R1=2.;
R2=4.;

//Sampling frequency
Freq=500;
//Step of sampling
pas=1/Freq;

//The system is of the form X''(t) = -a*X(t-R1) + b*u(t-R2)
a=2.7;
b=1.5;

//Choice of an output X
function x=X(t)
  x=3*sin(t/2)+2*cos(t/3)
endfunction

//The corresponding control
function u=U(t)
 u=(a*(3*sin((t-R1+R2)/2)+2*cos((t-R1+R2)/3))-(3*sin((t+R2)/2)/4+2*cos((t+R2)/3)/9))/b
endfunction

//Used to evaluate the computation time
dat=getdate();
date1=(dat(7)*60+dat(8))*60+dat(9)+dat(10)/1000;

//We will compute the mean and standard deviations of our evaluations
//using many successive tests
RESU=[];
for J=1:100 do

J

//We use a gaussian noise
rand('normal');
noise=zeros(1,floor(TFIN/pas)+1);
for i=1:(floor(TFIN/pas)+1)
  noise(1,i)=rand();
end;


//Noise level
amplitudeBruit=0.2;

TT=TDEBUT:pas:TFIN;
XX=X(TT);
UU=U(TT);

XX=XX+amplitudeBruit*noise;


//xset("window",1)
//xbasc()
//xname("Sortie")
//plot2d([TDEBUT:pas:TFIN],XX)

//computing the solution
RESU=[twoDelaysIdentification(XX,UU,Freq,15.,2.,10.,20,1.8,3.8,10);RESU];

end;

dat=getdate();

//Dure de l'excution
(dat(7)*60+dat(8))*60+dat(9)+dat(10)/1000 - date1

//Computing the means
mean(RESU,'r')

//computing the standard deviations
variance(RESU,'r').^(0.5)

//////////////////////////////////////////////////////////////////////////////////
// Some examples of results 
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.01; T00=15.; T=10.
// dure excution :    153.21
// moyennes des paramtres: !   2.699776    1.4999012    1.9993592    3.9994736 !
// moyennes des paramtres: !   0.0052367    0.0029861    0.0020830    0.0022692 !
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.025; T00=15.; T=10.   
// dure excution :    150.216
// moyennes des paramtres: !   2.6999443    1.5000033    1.9996908    3.999779 !
// moyennes des paramtres: !   0.0120262    0.0068887    0.0047869    0.0051869 !     
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.05; T00=15.; T=10.   
// dure excution :    149.735
// moyennes des paramtres: !   2.697788    1.4987364    1.999561    3.9997953 !
// moyennes des paramtres: !   0.0185222    0.0105852    0.0101759    0.0111249 !                 
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.1; T00=15.; T=10.
// dure excution :   149.745
// moyennes des paramtres: !   2.7015942    1.500871    1.997579    3.9976715 !
// carts types:            !   0.0434568    0.0250035    0.0165464    0.0178877 !        
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.1; T00=5.; T=7.
// dure excution :   111.641
// moyennes des paramtres: !   2.6958782    1.4977314    1.9995962    4.0000641 !
// carts types:            !   0.0522835    0.0299784    0.0193226    0.0212025 !      
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.2; T00=15.; T=10.   
// dure excution :   149.706
// moyennes des paramtres: !   2.6815971    1.4893259    1.999527    4.0008044 !
// carts types:            !   0.0952689    0.0545265    0.0396904    0.0435476 !             
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.2; T00=5.; T=7.
// dure excution:    111.641
// moyennes des paramtres: !   2.6847153    1.4914105    2.0002316    4.0013817 !
// carts types:            !   0.0930219    0.0535037    0.0382681    0.0411998 !      
//////////////////////////////////////////////////////////////////////////////////
//amplitudeBruit:  0.3; T00=15.; T=10.
// dure excution:    166.78
// moyennes des paramtres: !   2.6618239    1.4780899    2.0008394    4.0035609 !
// carts types:            !   0.1402121    0.0804748    0.0547170    0.0594828 !        
//////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 0.4 T00=15.; T=10.
// dure excution:    166.669
// moyennes des paramtres: !   2.5834088    1.4336471    1.9531993    3.9579016 !
// carts types:            !   0.5627448    0.3209027    0.5276842    0.5272897 !     
//////////////////////////////////////////////////////////////////////////////////
