// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file may be used with scilab 2.0 and 3.0
// Computation times are much longer with scilab 4.0 and the results sometimes strange...
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file corresponds to Example 5 Fig. 3
// Relax. Computation takes more than 50 min on my computer...
////////////////////////////////////////////////////////////////////


//To evaluate the computation time
//Dure de l'excution
dat=getdate();
date1=(dat(7)*60+dat(8))*60+dat(9);


//Identification of the parameters of a linear differential system
//with a varying delay.

//Order of the system
n=2;


//Number of integrals used
//Minimum is n+2
p=(n+2)+1;

//Number of integrals to be computed + 1 (first is index 0).

m=n+p;



//Begin and end of simulation
TFIN=130;
TDEBUT=0.;
//T_0 in the paper. After that time, we begin to move 
//the estimation of the delay
TRetInt=40;

//The delay function
function retard=Retard(t)
  retard= -4 + 0.01*t
endfunction;

//Constants \lambda, \Lambda and \lambda_{h} in the paper.
lambda=1.;
lambdaL=0.0001;
lambdaR=0.4;

//Not used functionality
Factor=1;


//Saving step
pas=0.01;

//We chose the coefficients so that the system is stable.
L2=-[0.5,0.7];
aa2=-coeff(poly(L2,"s"));
bb2=2;

//Initial condition for x
x0=[20.;0.3];


//We could chose varying coefficients too.
function aa=AA(t) 
  aa=aa2(1:2);
endfunction 

function bb=BB(t)
  bb=bb2
endfunction

//The control.
function u=U(t)
 u=60.0*cos(1.23*t+1.3*sin(t)-0.7*cos(0.5*t))
endfunction

//Choice of the noise
//A gaussian noise
amplitudeG=5.0;
pasG=pas;
rand('normal');
for i=1:(floor(TFIN/pasG)+1)
  RandNums(i)=rand();
end;
//We may had a high frequency cos function if we like
amplitudeS=0.;
frequenceB=500;
function noise=NOISE(t)
  noise=amplitudeG*RandNums(floor(t/pasG)+1)+amplitudeS*cos(frequenceB*t)
endfunction


//Matrix allowing to express the integral of x^(i)
M0=zeros(n+1,m);
M0(1,1)=1;
MM=M0;
//Attention i=1 correspond  la convolution avec exp(-lambda t)
//          i=2                                  t exp(-lambda t)
//          i=3                                  t^2 exp(-lambda t)
for i=2:m
  M1=[zeros(1,i-1),1,zeros(1,m-i)];
  for j=2:n+1
    M1=[M1;-lambda*M1(j-1,:)+(i-1)*M0(j-1,:)];
  end;
  M0=M1;
  MM=[MM;M1];
end;

//We build a differential system to do everything :
//To compute x ;
//To compute the integrals I_{f_{j},x}
//XIxIuAB=[x1,...,xn,Ix0,...,Ixn+p,Iu0,...,Iun+p,a0,...,an-1, b1      b2      R]
//          1      n   n+1     n+m   n+m+1 n+2m, n+2m+1 2n+2m 2n+2m+1 2n+2m+2 2n+2m+3
//And to solve the linear system by a gradient method.
function dxdixdiudabdr=DXDIDABR(t,XIxIuABR)
//  if floor(t)==t then 
//    disp(t) 
//  end;
  xx=XIxIuABR(1:n);
  aa=AA(t);
  bb=BB(t);
  dx=xx(2:n);
  ab=XIxIuABR(n+2*m+1:2*n+2*m+2);
  R=XIxIuABR(2*n+2*m+3);
//The linear system of order n
// dx(i) = x(i+1) si i<n
// dx(n) a0*x0 + ... + an-1*xn-1
//Be carrefull, a0 corresponds to aa(1) !
  dx=[dx; aa*xx(:)+bb*U(t+Retard(t))];
//The system whose solutions tend to 
//\int_{0}^{t}\tau^i*exp(-lambda*\tau)x(T-\tau) d\tau.
  dix=[xx(1)+cur_noise; [1:m-1]'.*XIxIuABR((n+1):(n+m-1))]-lambda*XIxIuABR((n+1):(n+m));
  diu=[U(t+R); [1:m-1]'.*XIxIuABR((n+m+1):(n+2*m-1))]-lambda*XIxIuABR((n+m+1):(n+2*m));
  C=[];
  for i=n+1:m
    C=[C,MM((i-1)*(n+1)+1:i*(n+1),1:m)*XIxIuABR((n+1):(n+m),1)];
  end;
  CC=C(1:n,1:m-n);CC=CC';
  CC=[CC,XIxIuABR(2*n+m+1:n+2*m,1),XIxIuABR(2*n+m:n+2*m-1,1).*[n+1:m]'];
  DD=C(n+1,:)';
  dab=-lambdaL*CC'*(CC*ab-DD);
  if t< TRetInt then,
    dxdixdiudabdr=[dx;Factor*dix;Factor*diu;Factor*dab;0];
                else
    dr=lambdaR*XIxIuABR(2*n+2*m+2)/XIxIuABR(2*n+2*m+1)
    dxdixdiudabdr=[dx;Factor*dix;Factor*(1+dr)*diu;Factor*dab;Factor*dr];
  end
endfunction


lastresu=[x0;zeros(2*m+n+2,1);-3.6];
resu=lastresu;
for t0=TDEBUT+pas:pas:TFIN do
  cur_noise=NOISE(t0);
  lastresu=ode(lastresu,t0-pas,[t0],DXDIDABR);
  resu=[resu lastresu];
end;

xset("window",1)
xbasc()
xname("X")
plot2d([TDEBUT:pas:TFIN]', [resu(1,:)'+NOISE([TDEBUT:pas:TFIN]') resu(1,:)'],..
  [1,4],"111","Noisy output@Real x",[0,-250,TFIN,250])


a1reel=[];
for t=TDEBUT:pas:TFIN do
 aaa=AA(t);
 a1reel=[a1reel aaa(2)];
end;
xset("window",2)
xbasc()
xname("Estimations of coefficients")
plot2d([TDEBUT:pas:TFIN]',[resu(n+2*m+1,:)' resu(n+2*m+2,:)' resu(n+2*m+3,:)' resu(n+2*m+4,:)'],..
  [1,2,3,4],"111","Estimated a0@Estimated a1@Estimated b@b2",[0,-3,TFIN,3])

//Transposition of a Maple computation (retarddiff2)..
xEvalue=-((-resu(n+2*m+2,:)-2*lambda).*resu(n+1,:)+..
          (-resu(n+2*m+1,:)+resu(n+2*m+2,:)*lambda+lambda^2).*resu(n+2,:)..
           -resu(2*n+2*m+1,:).*resu(n+m+2,:));

dxEvalue= -((-resu(n+2*m+1,:)+resu(n+2*m+2,:)*lambda+lambda^2).*resu(n+1,:)..
                    +(-lambda-resu(n+2*m+2,:)).*xEvalue-resu(2*n+2*m+1,:).*resu(n+m+1,:));


xset("window",3)
xbasc()
xname("Estimation of x")
plot2d([TDEBUT:pas:TFIN]',[xEvalue' resu(1,:)'],..
  [1,2],"111","Estimated x@Real x",[0,-250,TFIN,250])
xset("window",4)
xbasc()
xname("Estimation of x derivative")
plot2d([TDEBUT:pas:TFIN]',[dxEvalue' resu(2,:)'],..
  [1,2],"111","Estimated dx@Real dx",[0,-130,TFIN,130])

retardreel=[];
for t=TDEBUT:pas:TFIN do
  retardreel=[retardreel Retard(t)];
end;
xset("window",5)
xbasc()
xname("Estimation of delay")
plot2d([TDEBUT:pas:TFIN]',-[resu(n+2*m+5,:)' retardreel'],..
  [1,2],"111","Estimated delay@Real delay",[0,2.55,TFIN,4.05])


dat=getdate();

//Dure de l'excution. Execution time
(dat(7)*60+dat(8))*60+dat(9) - date1


//resu(:,13001)
//!   121.20675 !
//!   1.2199636 !
//!   158.3377  !
//!   181.69156 !
//!   348.12425 !
//!   863.42595 !
//!   2502.8074 !
//!   7680.339  !
//!   19283.117 !
//!   23.651017 !
//!   10.204044 !
//!   39.908974 !
//!   194.89277 !
//!   900.07323 !
//!   4202.6715 !
//!   20462.233 !
//! - 0.3491681 !
//! - 1.1998981 !
//!   1.9231932 !
//!   0.0591302 !
//! - 2.7287475 !                                                                                                         
//xEvalue(13001)
//   119.18954  
//dxEvalue(13001)
// - 1.9753557  
//Retard(130)
// - 2.7 